#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys
import math
import time
import numpy as np
import matplotlib.pyplot as plt
from collections import deque

sys.path.append('environment')
sys.path.append('samplingplanner')
 
import Environment as env
import SamplingPlanner as sp
import RRTDubinsPlanner as rrt

SLACK = 0.975

if __name__ == "__main__":
    #define the planning scenarios
    #scenario name
    #start configuration
    #goal configuration
    #limits for individual DOFs
    #turning radius

    scenarios_rrt = [("environments/simple_test", (2,2,0,0,0,0), (-2,-2,0,0,0,math.pi), [(-3,3), (-3,3), (0,0), (0,0), (0,0), (0,2*math.pi)], 1.),
                     ("environments/simple_test", (2,2,0,0,0,0), (-2,-2,0,0,0,math.pi/2), [(-3,3), (-3,3), (0,0), (0,0), (0,0), (0,2*math.pi)], .5),
                     ("environments/squares", (1,-2,0,0,0,math.pi/2), (4,7,0,0,0,3*math.pi/2), [(0,5), (-4,9), (0,0), (0,0), (0,0), (0,2*math.pi)], 1.),
                   ]

    #####################################
    ## EVALUATION OF THE RRT PLANNER
    #####################################
    for scenario in scenarios_rrt:
        name = scenario[0]
        start = sp.te_2_se3(scenario[1])
        goal = sp.te_2_se3(scenario[2])
        limits = scenario[3]
        turning_radius = scenario[4]
        
        print("processing scenario: " + name)

        #initiate environment and robot meshes
        environment = env.Environment()
        environment.load_environment(name)
        ax = environment.plot_environment(start, ax=None, limits=limits)
        plt.pause(0.00001)

        #instantiate the planner
        planner = rrt.RRTPlanner(limits, turning_radius)
        planner_slack = rrt.RRTPlanner(limits, turning_radius*SLACK)
    
        #plan the path through the environment
        path = planner.plan(environment, start, goal)

        ax = environment.plot_environment(path[0], ax=ax,  limits=limits)
        planner.plot_samples(plot_samples = False, plot_edges = False)
        for i in range(len(path)-1):
            print ('%5d / %5d' % (i,len(path)))

            se3_1 = path[i]
            se3_2 = path[i+1]
            #for se3_j in planner.create_sub_path(se3_1,se3_2,slack=SLACK):
            for se3_j in planner_slack.create_sub_path(se3_1,se3_2):
                ax = environment.plot_environment(se3_j, ax=ax,  limits=limits)
                planner.plot_samples(plot_samples = False, plot_edges = False)
                plt.pause(0.00001)

            if planner_slack.is_valid_edge(se3_1,se3_2,environment,verbose=True):
                print ('slack ok')
            else:
                print ('slack fail')
                break

